@@ -144,6 +144,7 @@ perl_ranlib=`$perl_interp -MConfig -e 'print $Config{ranlib}'`
 #   determine static objects
 #
 perl_static_exts="$param_PERL_STATIC_EXTS"
+perl_static_ar="$param_PERL_STATIC_AR"
 perl_static_srcs="$param_PERL_STATIC_SRCS"
 perl_static_objs="`echo $param_PERL_STATIC_SRCS | sed -e 's:\.c:.o:g'`"
 perl_static_objs_pic="`echo $param_PERL_STATIC_SRCS | sed -e 's:\.c:.lo:g'`"
@@ -154,7 +155,7 @@ perl_static_objs_pic="`echo $param_PERL_STATIC_SRCS | sed -e 's:\.c:.lo:g'`"
 perl_defs=''
 perl_defs="$perl_defs -DMOD_PERL_VERSION=\\\"$param_MOD_PERL_VERSION\\\""
 perl_defs="$perl_defs -DMOD_PERL_STRING_VERSION=\\\"mod_perl/$param_MOD_PERL_VERSION\\\""
-perl_defs="$perl_defs -DPERL_VERSION=$param_PERL_VERSION"
+perl_defs="$perl_defs -DPERLV=$param_PERLV"
 OIFS="$IFS" IFS="$DIFS"
 for hook in \
     DISPATCH CHILD_INIT CHILD_EXIT POST_READ_REQUEST TRANS HEADER_PARSER \
@@ -189,12 +190,13 @@ echo "PERL_INC=$perl_inc"
 echo "PERL_LD=$perl_ld"
 echo "PERL_LDFLAGS=$perl_ldflags"
 echo "PERL_LDDLFLAGS=$perl_lddlflags"
-echo "PERL_LIBS=$perl_libs"
+echo "PERL_LIBS=$perl_libs $perl_static_ar"
 echo "PERL_XSINIT=$perl_xsinit"
 echo "PERL_XSUBPP=$perl_xsubpp"
 echo "PERL_AR=$perl_ar"
 echo "PERL_RANLIB=$perl_ranlib"
 echo "PERL_STATIC_EXTS=$perl_static_exts"
+echo "PERL_STATIC_AR=$perl_static_ar"
 echo "PERL_STATIC_SRCS=$perl_static_srcs"
 echo "PERL_STATIC_OBJS=$perl_static_objs"
 echo "PERL_STATIC_OBJS_PIC=$perl_static_objs_pic"
