@@ -88,7 +88,7 @@ static command_rec perl_cmds[] = {
     { "</Perl>", perl_end_section, NULL, OR_ALL, NO_ARGS, "End Perl code" },
 #endif
     { "=pod", perl_pod_section, NULL, OR_ALL, RAW_ARGS, "Start of POD" },
-    { "=end", perl_pod_section, NULL, OR_ALL, RAW_ARGS, "End of =begin" },
+    { "=back", perl_pod_section, NULL, OR_ALL, RAW_ARGS, "End of =over" },
     { "=cut", perl_pod_end_section, NULL, OR_ALL, NO_ARGS, "End of POD" },
     { "__END__", perl_config_END, NULL, OR_ALL, RAW_ARGS, "Stop reading config" },
     { "PerlFreshRestart", perl_cmd_fresh_restart,
@@ -433,6 +433,54 @@ static void mp_dso_unload(void *data)
 } 
 #endif
 
+static void mp_server_notstarting(void *data) 
+{
+    saveINC;
+    perl_require_module("Apache", NULL); 
+    Apache__ServerStarting(FALSE);
+}
+
+#define Apache__ServerStarting_on() \
+    Apache__ServerStarting(PERL_RUNNING()); \
+    register_cleanup(p, NULL, mp_server_notstarting, mod_perl_noop) 
+
+#define MP_APACHE_VERSION 1.25
+
+void mp_check_version(void)
+{
+    I32 i;
+    SV *namesv;
+    SV *version = perl_get_sv("Apache::VERSION", FALSE);
+
+    if(!version)
+	croak("Apache.pm failed to load!"); /*should never happen*/
+    if(SvNV(version) >= MP_APACHE_VERSION) /*no worries*/
+	return;
+
+    fprintf(stderr, "Apache.pm version %.02f or higher required!\n", 
+	    MP_APACHE_VERSION);
+    fprintf(stderr, "%s", form("%_ is only version %_\n", 
+			       *hv_fetch(GvHV(incgv), "Apache.pm", 9, FALSE),
+			       version));
+    fprintf(stderr, 
+	    "Perhaps you forgot to 'make install' or need to uninstall an old version?\n");
+
+    namesv = NEWSV(806, 0);
+    for(i=0; i<=AvFILL(GvAV(incgv)); i++) {
+	char *tryname;
+	PerlIO *tryrsfp = 0;
+	SV *dir = *av_fetch(GvAV(incgv), i, TRUE);
+	sv_setpvf(namesv, "%_/Apache.pm", dir);
+	tryname = SvPVX(namesv);
+	if((tryrsfp = PerlIO_open(tryname, "r"))) {
+	    fprintf(stderr, "Found: %s\n", tryname);
+	    PerlIO_close(tryrsfp);
+	}
+    }
+    SvREFCNT_dec(namesv);
+    exit(1);
+}
+
 void perl_startup (server_rec *s, pool *p)
 {
     char *argv[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
@@ -449,6 +497,11 @@ void perl_startup (server_rec *s, pool *p)
 #include "mod_perl_version.h"
 #endif
     ap_add_version_component(MOD_PERL_STRING_VERSION);
+    if(PERL_RUNNING()) {
+	if(perl_get_sv("Apache::Server::AddPerlVersion", FALSE)) {
+	    ap_add_version_component(form("Perl/%s", patchlevel));
+	}
+    }
 #endif
 
 #ifndef WIN32
@@ -476,12 +529,18 @@ void perl_startup (server_rec *s, pool *p)
     dstr = NULL;
 #endif
 
+    if(PERL_RUNNING()) {
+	saveINC;
+	mp_check_version();
+    }
+    
     if(perl_is_running == 0) {
 	/* we'll boot Perl below */
     }
     else if(perl_is_running < PERL_DONE_STARTUP) {
 	/* skip the -HUP at server-startup */
 	perl_is_running++;
+	Apache__ServerStarting_on();
 	MP_TRACE_g(fprintf(stderr, "perl_startup: perl aleady running...ok\n"));
 	return;
     }
@@ -575,7 +634,7 @@ void perl_startup (server_rec *s, pool *p)
     (void)GvHV_init("mod_perl::UNIMPORT");
 
     Apache__ServerReStarting(FALSE); /* just for -w */
-    Apache__ServerStarting(PERL_RUNNING());
+    Apache__ServerStarting_on();
 
 #ifdef PERL_STACKED_HANDLERS
     if(!stacked_handlers) {
@@ -615,7 +674,6 @@ void perl_startup (server_rec *s, pool *p)
 			   "mod_perl: PerlModule,PerlRequire postponed\n"));
 	my_setenv("PERL_STARTUP_DONE", "1");
 	saveINC;
-	Apache__ServerStarting(FALSE);
 	return;
     }
 
@@ -651,10 +709,9 @@ void perl_startup (server_rec *s, pool *p)
 #endif
 
     saveINC;
-    Apache__ServerStarting(FALSE);
 #if MODULE_MAGIC_NUMBER >= MMN_130
     if(perl_module.dynamic_load_handle) 
-	register_cleanup(p, NULL, mp_dso_unload, NULL); 
+	register_cleanup(p, NULL, mp_dso_unload, null_cleanup); 
 #endif
 }
 
@@ -1381,19 +1438,6 @@ callback:
 	MP_STORE_ERROR(r->uri, ERRSV);
 	if(!perl_sv_is_http_code(ERRSV, &status))
 	    status = SERVER_ERROR;
-#if MODULE_MAGIC_NUMBER >= MMN_130
-	if(!SvREFCNT(TOPs)) {
-#ifdef WIN32
-	    mod_perl_error(r->server,
-			   "mod_perl: stack is corrupt, server may need restart\n");
-#else
-	    mod_perl_error(r->server,
-			   "mod_perl: stack is corrupt, exiting process\n");
-	    my_setenv("PERL_DESTRUCT_LEVEL", "-1");
-	    child_terminate(r);
-#endif /*WIN32*/
-	}
-#endif
     }
     else if(count != 1) {
 	mod_perl_error(r->server,
